Index: include/asterisk/threadstorage.h
===================================================================
--- include/asterisk/threadstorage.h	(revision 233781)
+++ include/asterisk/threadstorage.h	(working copy)
@@ -104,6 +104,7 @@
 #define AST_THREADSTORAGE_CUSTOM(a,b,c)	AST_THREADSTORAGE_CUSTOM_SCOPE(a,b,c,static)
 
 #if !defined(DEBUG_THREADLOCALS)
+#if !defined(SOLARIS)
 #define AST_THREADSTORAGE_CUSTOM_SCOPE(name, c_init, c_cleanup, scope)	\
 static void __init_##name(void);                \
 scope struct ast_threadstorage name = {         \
@@ -115,7 +116,21 @@
 {                                               \
 	pthread_key_create(&(name).key, c_cleanup); \
 }
+#else /* defined(SOLARIS) */
+#define AST_THREADSTORAGE_CUSTOM_SCOPE(name, c_init, c_cleanup, scope)	\
+static void __init_##name(void);                \
+scope struct ast_threadstorage name = {         \
+	.once = {PTHREAD_ONCE_INIT},                  \
+	.key_init = __init_##name,                  \
+	.custom_init = c_init,                      \
+};                                              \
+static void __init_##name(void)                 \
+{                                               \
+	pthread_key_create(&(name).key, c_cleanup); \
+}
+#endif /* defined(SOLARIS) */
 #else /* defined(DEBUG_THREADLOCALS) */
+#if !defined(SOLARIS)
 #define AST_THREADSTORAGE_CUSTOM_SCOPE(name, c_init, c_cleanup, scope) \
 static void __init_##name(void);                \
 scope struct ast_threadstorage name = {         \
@@ -132,6 +147,24 @@
 {                                               \
 	pthread_key_create(&(name).key, __cleanup_##name); \
 }
+#else /* defined(SOLARIS) */
+#define AST_THREADSTORAGE_CUSTOM_SCOPE(name, c_init, c_cleanup, scope) \
+static void __init_##name(void);                \
+scope struct ast_threadstorage name = {         \
+	.once = {PTHREAD_ONCE_INIT},                  \
+	.key_init = __init_##name,                  \
+	.custom_init = c_init,                      \
+};                                              \
+static void __cleanup_##name(void *data)        \
+{                                               \
+	__ast_threadstorage_object_remove(data);    \
+	c_cleanup(data);                            \
+}                                               \
+static void __init_##name(void)                 \
+{                                               \
+	pthread_key_create(&(name).key, __cleanup_##name); \
+}
+#endif /* defined(SOLARIS) */
 #endif /* defined(DEBUG_THREADLOCALS) */
 
 /*!
Index: res/res_musiconhold.c
===================================================================
--- res/res_musiconhold.c	(revision 233781)
+++ res/res_musiconhold.c	(working copy)
@@ -43,6 +43,10 @@
 #include <dirent.h>
 #include <sys/ioctl.h>
 
+#ifdef SOLARIS
+#include <thread.h>
+#endif
+
 #ifdef HAVE_DAHDI
 #include <dahdi/user.h>
 #endif
